// -*- C++ -*-
//
// This is the implementation of the non-inlined, non-templated member
// functions of the Hw64Selector class.
//

#include "Hw64Selector.h"
#include "ThePEG/Interface/ClassDocumentation.h"
#include "ThePEG/Persistency/PersistentOStream.h"
#include "ThePEG/Persistency/PersistentIStream.h"
#include "ThePEG/Repository/UseRandom.h"
#include "CheckId.h"
#include "Herwig++/Utilities/Kinematics.h"

using namespace Herwig;

void Hw64Selector::persistentOutput(PersistentOStream & ) const {
}

void Hw64Selector::persistentInput(PersistentIStream & , int) {
}

ClassDescription<Hw64Selector> Hw64Selector::initHw64Selector;
// Definition of the static class description member.

void Hw64Selector::Init() {

  static ClassDocumentation<Hw64Selector> documentation
    ("There is no documentation for the Hw64Selector class");

}

pair<tcPDPtr,tcPDPtr> Hw64Selector::chooseHadronPair(const Energy cluMass,
						     const long id1,
						     const long id2,const long)
  throw(Veto, Stop, Exception) {
  bool diquark = !(DiquarkMatcher::Check(id1) || DiquarkMatcher::Check(id2));



  pair<tcPDPtr,tcPDPtr> lighthad = lightestHadronPair(id1,id2);
  if(!lighthad.first || !lighthad.second)
    throw Exception() << "Hw64Selector::chooseHadronPair "
		      << "We have 0's! First id = " << id1 << " second = " 
		      << id2 << ". This is probably a problem with either"
		      << " undecayed heavy particles or colour connections" 
		      << Exception::eventerror;
  // calculate maximum momentum
  Energy PCMax = Kinematics::pstarTwoBodyDecay(cluMass,lighthad.first->mass(),
					       lighthad.second->mass());
  tcPDPtr had1=tcPDPtr(),had2=tcPDPtr();
  int ntry = 0;
  int idQ = -123456789;
  const int nmax = 5000;
  Energy p;
  do {
    idQ=partons()[UseRandom::irnd(partons().size())];
    if(diquark&&DiquarkMatcher::Check(idQ)) continue;
    KupcoData::iterator it1,it2;
    if(pwt()[idQ] <=UseRandom::rnd()) continue;
    pair<long,long> pid(abs(id1),idQ);
    do {
      it1=table()[pid].begin();
      advance(it1,int(double(table()[pid].size())*UseRandom::rnd()));
    } 
    while(it1!=table()[pid].end() && it1->overallWeight < UseRandom::rnd());
    had1 = it1->ptrData;
    pid=make_pair(idQ,abs(id2));
    do {
      it2=table()[pid].begin();
      advance(it2,int(double(table()[pid].size())*UseRandom::rnd()));
    }
    while(it2!=table()[pid].end() && it2->overallWeight < UseRandom::rnd());
    had2=it2->ptrData;
    if(had1 && had2) {
      p = Kinematics::pstarTwoBodyDecay(cluMass, it1->mass, it2->mass);
      if(p/PCMax < UseRandom::rnd()) { 
	had1 = had2 = tcPDPtr(); 
	ntry++;
      }
    }
  }
  while((!had1|| !had2) && ntry < nmax);
  if(ntry >= nmax) return lighthad;
  int signQ = 0, signHad1 = -12345678, signHad2 = -12345678;
  if((CheckId::canBeMeson(id1,-idQ) || CheckId::canBeBaryon(id1,-idQ)) &&
     (CheckId::canBeMeson(idQ,id2)  || CheckId::canBeBaryon(idQ,id2))) 
	  signQ = +1;
  if((CheckId::canBeMeson(id1,idQ ) || CheckId::canBeBaryon(id1,idQ)) &&
     (CheckId::canBeMeson(-idQ,id2) || CheckId::canBeBaryon(-idQ,id2)))
	  signQ = -1;
  if(signQ) {
    signHad1 = signHadron(id1, -signQ*idQ, had1);
    signHad2 = signHadron(id2,  signQ*idQ, had2);
  } 
  else throw Exception() << "Hw64Selector::chooseHadronPair()"
			 << Exception::abortnow;
  return make_pair( signHad1 > 0 ? had1 : tcPDPtr(had1->CC()),
		    signHad2 > 0 ? had2 : tcPDPtr(had2->CC()));
}

    

